#
# hci.py
#
# Copyright (c) 2018 Virtuozzo International GmbH. All rights reserved.
#
# This file is part of OpenVZ. OpenVZ is free software;
# you can redistribute it and/or modify it under the terms of the GNU
# Lesser General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Virtuozzo International GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.

from pyanaconda.constants import THREAD_CUSTOM_STORAGE_INIT, THREAD_STORAGE, THREAD_STORAGE_WATCHER
from pyanaconda.i18n import N_, _
from pyanaconda.product import productName
from pyanaconda.threads import threadMgr
from pyanaconda.ui.lib.disks import getDisks
from pyanaconda.virtuozzo_c import inside_vm
from pyanaconda.iutil import expert_mode
from blivet.size import Size
from blivet.util import total_memory
from distutils.sysconfig import get_python_lib
import os
import sys

sys.path.insert(0,"%s/pyanaconda/installclasses" % get_python_lib(plat_specific=1))

from virtuozzo import VirtuozzoBaseInstallClass

class VzHCIBaseInstallClass(VirtuozzoBaseInstallClass):
    vendor = "Virtuozzo"
    name = N_("%s Infrastructure Platform" % vendor)
    hidden = False
    if productName != "%s Infrastructure Platform" % vendor:
        hidden = True

    def __init__(self):
        VirtuozzoBaseInstallClass.__init__(self)
        self.is_vz = False
        self.with_va = False
        self.create_bridges = False
        self.fix_ifcfgs = True
        self.select_minsize_slash = False
        self.slash_size = Size("20GiB")
        # 0 means default value - see blivet sources
        self.max_slash_size = 0
        self.mask_nm = True
        self.docker_img_ver = "4.0.3"
        self.prohibited_net_devs = ("team","bridge",)
        self.ntp_always_on = True
        self.min_ram = "2GiB"
        self.reserved_ram = "256MiB"
        self.min_cpus = 2
        self.min_net_speed = 1000
        self.min_net_speed_gb = self.min_net_speed/1000 # E.g. 1GBit
        self.net_speed = 0
        self.net_devs = 0
        self.prohibit_installation = False

    @property
    def not_ask_reboot(self):
        return not expert_mode()

    @property
    def min_hdd_size(self):
        # 100 GB, see #VSTOR-12771
        return Size("100GB")

    @property
    def get_eula(self):
        return "/usr/share/licenses/hci-release/EULA"

    @property
    def get_release_f(self):
        return "/etc/hci-release"

    def min_req_warn(self, mem, cpu, disks, net):
        with open("/proc/cpuinfo", "r") as f:
            num_cpus = len([c for c in f.readlines() if c.startswith("processor")])
            if num_cpus < self.min_cpus:
                cpu.set_text(_("At least %i logical CPUs are required. You have %i installed.") % \
                    (self.min_cpus, num_cpus))
                cpu.set_visible(True)

        if total_memory() < (Size(self.min_ram) - Size(self.reserved_ram)):
            mem.set_text(_("At least %s of RAM are required. You have %s installed.") % \
                    (Size(self.min_ram), total_memory()))
            mem.set_visible(True)

        text = _("At least one hard disk with the capacity of %iGB is required.") % self.min_hdd_size.convertTo("GB")
        for t in (THREAD_CUSTOM_STORAGE_INIT, THREAD_STORAGE, THREAD_STORAGE_WATCHER):
            threadMgr.wait(t)
        try:
            lagest_hdd_size = sorted([ d for d in getDisks(self.anaconda.storage.devicetree) if not d.protected ], \
                    key = lambda k: k.size, reverse = True)[0].size
            if self.min_hdd_size > lagest_hdd_size:
                disks.set_text(text + _(" Your largest disk is only %iGB.") % lagest_hdd_size.convertTo("GB"))
                disks.set_visible(True)
                self.prohibit_installation = True
        except:
            disks.set_text(text + _(" No hard disks found."))
            disks.set_visible(True)
            self.prohibit_installation = True

        sysnet = "/sys/class/net"
        for i in os.listdir(sysnet):
            try:
                with open("%s/%s/address" % (sysnet, i), "r") as f:
                    if len(f.read()) < 18: # 18 - default MAC addr len
                        continue
                self.net_devs += 1
                with open("%s/%s/speed" % (sysnet, i), "r") as f:
                    self.net_speed = max(int(f.read()), self.net_speed) # Speed mentioned here is in MBit
            except:
                pass

        if not inside_vm() and self.min_net_speed > self.net_speed:
            text = _("At least %i Gbps network adapter is required.") % self.min_net_speed_gb
            if self.net_speed:
                text += _(" You have %i Mbps adapter(s) installed.") % self.net_speed
            elif not self.net_devs:
                text += _(" Failed to detect your network adapters.")
            else:
                text += _(" Failed to detect your network adapter speed.")
            net.set_text(text)
            net.set_visible(True)

        return [ req for req in (mem, cpu, disks, net) if req.get_property("visible") ]
